/*
 * CameraContext.h
 *
 *  Created on: Jul 9, 2010
 *      Author: petrito1@cmp.felk.cvut.cz
 */

#ifndef NIFTI_CAMERA_CAMERACONTEXT_H_
#define NIFTI_CAMERA_CAMERACONTEXT_H_

#include <vector>

#include "Camera.h"
#include "LadybugCamera.h"

namespace nifti {
namespace dc1394 {

/**
 * RAII-based camera context using libdc1394.
 * Cameras created in this context cannot be used after its deallocation.
 */
class CameraContext {
public:
  /**
   * Initializes a dc1394_t instance.
   *
   * @throws CameraException
   */
  CameraContext();

  /**
   * Frees the dc1394_t instance and all initialized dc1394camera_t instances.
   */
  ~CameraContext();

  /**
   * Returns the first camera.
   *
   * @throws CameraException
   */
  Camera getCamera();

  /**
   * Returns the first Ladybug camera.
   * FIXME: Assumes the first camera is Ladybug.
   */
  LadybugCamera getLadybugCamera(uint32_t serialNumber);
private:

  /**
   * Camera context to be freed by dc1394_free in the destructor.
   */
  dc1394_t* context;

  /**
   * Cameras to be freed by dc1394_camera_free in the destructor.
   */
  std::vector<dc1394camera_t*> cameras;

  // Prevent copying and assignment - intentionally not implemented.
  CameraContext(const CameraContext&);
  CameraContext& operator=(const CameraContext&);

  dc1394camera_t* getDc1394Camera(uint32_t serialNumber = 0);
};

}
}

#endif /* NIFTI_CAMERA_CAMERACONTEXT_H_ */
