/*
 * CameraException.h
 *
 *  Created on: Jul 28, 2010
 *      Author: petrito1@cmp.felk.cvut.cz
 */

#ifndef NIFTI_CAMERA_CAMERAEXCEPTION_H_
#define NIFTI_CAMERA_CAMERAEXCEPTION_H_

#define CAMERA_EXCEPTION(err, message)                    \
  do {                                                    \
    if ((err>0)||(err<=-DC1394_ERROR_NUM))                \
      err=DC1394_INVALID_ERROR_CODE;                      \
                                                          \
    if (err!=DC1394_SUCCESS) {                            \
      dc1394_log_error("%s: in %s (%s, line %d): %s\n",   \
      dc1394_error_get_string(err),                       \
          __FUNCTION__, __FILE__, __LINE__, message);     \
      throw nifti::dc1394::CameraException(err, message); \
    }                                                     \
  } while (0);

#include <stdexcept>
#include <string>

#include <dc1394/dc1394.h>

namespace nifti {
namespace dc1394 {

/**
 * Camera exception, may wrap a dc1394error_t instance.
 */
class CameraException: public std::runtime_error {

public:
  /**
   * @param error a dc1394error_t error.
   * @param message a message describing context of where the exception occured.
   */
  CameraException(dc1394error_t error, std::string message);
  CameraException();
  virtual ~CameraException() throw ();

  dc1394error_t getError();
  std::string getErrorString();

private:
  dc1394error_t error;

};

}
}

#endif /* NIFTI_CAMERA_CAMERAEXCEPTION_H_ */
