/*
 * LadybugCamera.h
 *
 *  Created on: Sep 2, 2010
 *      Author: petrito1@cmp.felk.cvut.cz
 */

#ifndef NIFTI_DC1394_LADYBUGCAMERA_H_
#define NIFTI_DC1394_LADYBUGCAMERA_H_

#include "LadybugCamera.h"

#include "Camera.h"

using nifti::dc1394::Camera;

namespace nifti {
namespace dc1394 {

class LadybugCamera: public Camera {
public:
  static const int NUM_CAMERAS = 6;
  static const int NUM_BAYER_CHANNELS = 4;
  static const uint64_t IMAGE_SELECTION_REGISTER;
  static const uint32_t ALL_CAMERAS;
  static const uint32_t CAMERA_0;
  static const uint32_t CAMERA_1;
  static const uint32_t CAMERA_2;
  static const uint32_t CAMERA_3;
  static const uint32_t CAMERA_4;
  static const uint32_t CAMERA_5;
  static const uint32_t CAMERA_0_BAYER_CHANNEL_0;
  static const uint32_t CAMERA_0_BAYER_CHANNEL_1;
  static const uint32_t CAMERA_0_BAYER_CHANNEL_2;
  static const uint32_t CAMERA_0_BAYER_CHANNEL_3;
  static const uint32_t CAMERA_1_BAYER_CHANNEL_0;
  static const uint32_t CAMERA_1_BAYER_CHANNEL_1;
  static const uint32_t CAMERA_1_BAYER_CHANNEL_2;
  static const uint32_t CAMERA_1_BAYER_CHANNEL_3;
  static const uint32_t CAMERA_2_BAYER_CHANNEL_0;
  static const uint32_t CAMERA_2_BAYER_CHANNEL_1;
  static const uint32_t CAMERA_2_BAYER_CHANNEL_2;
  static const uint32_t CAMERA_2_BAYER_CHANNEL_3;
  static const uint32_t CAMERA_3_BAYER_CHANNEL_0;
  static const uint32_t CAMERA_3_BAYER_CHANNEL_1;
  static const uint32_t CAMERA_3_BAYER_CHANNEL_2;
  static const uint32_t CAMERA_3_BAYER_CHANNEL_3;
  static const uint32_t CAMERA_4_BAYER_CHANNEL_0;
  static const uint32_t CAMERA_4_BAYER_CHANNEL_1;
  static const uint32_t CAMERA_4_BAYER_CHANNEL_2;
  static const uint32_t CAMERA_4_BAYER_CHANNEL_3;
  static const uint32_t CAMERA_5_BAYER_CHANNEL_0;
  static const uint32_t CAMERA_5_BAYER_CHANNEL_1;
  static const uint32_t CAMERA_5_BAYER_CHANNEL_2;
  static const uint32_t CAMERA_5_BAYER_CHANNEL_3;

  static const uint32_t CAMERAS[NUM_CAMERAS];
  static const uint32_t BAYER_CHANNELS[NUM_CAMERAS][NUM_BAYER_CHANNELS];

  static const int RAW_WIDTH;
  static const int RAW_HEIGHT;
  static const int RAW_HALF_HEIGHT;
  static const int RAW_NUM_IMAGES;
  static const int MONO_WIDTH;
  static const int MONO_HEIGHT;
  static const int MONO_HALF_HEIGHT;
  static const int MONO_NUM_IMAGES;
  static void format7FrameInfo(const dc1394video_mode_t videoMode, int& frameWidth, int& frameHeight, int& imageWidth, int& imageHeight, int& numImages);

  LadybugCamera(dc1394camera_t* camera);
  LadybugCamera();
  virtual ~LadybugCamera();

  void setFormat7Roi(dc1394video_mode_t videoMode, int32_t packetSize = 9792);

  uint32_t getSelectedImages();
  void setSelectedImages(uint32_t images);
  bool areImagesSelected(uint32_t images);
  int getNumSelectedImages();
private:
  uint32_t selectedImages;
};

}
}

#endif /* NIFTI_DC1394_LADYBUGCAMERA_H_ */
