/*
 * CameraContext.cpp
 *
 *  Created on: Jul 9, 2010
 *      Author: petrito1@cmp.felk.cvut.cz
 */

#include "CameraContext.h"

#include <dc1394/dc1394.h>

#include "CameraException.h"

// Point Grey Research Inc. Organizationally Unique Identifier
//#define PGR_OUI 0x00B09D

namespace nifti {
namespace dc1394 {

CameraContext::CameraContext() {
  printf("CameraContext::CameraContext()\n");
  context = dc1394_new();
  if (!context) {
    throw CameraException(DC1394_SUCCESS, "Failed to initialize camera context.");
  }
}

CameraContext::~CameraContext() {
  printf("CameraContext::~CameraContext()\n");

  std::vector<dc1394camera_t*>::iterator iter;
  for (iter = cameras.begin(); iter < cameras.end(); iter++) {
    dc1394_iso_release_all(*iter);
    printf("dc1394_camera_free %s %s...\n", (*iter)->vendor, (*iter)->model);
    dc1394_camera_free(*iter);
  }
  printf("dc1394_free...\n");
  dc1394_free(context);
}

Camera CameraContext::getCamera() {
  printf("getCamera\n");

  Camera camera(getDc1394Camera());
  return camera;
}

LadybugCamera CameraContext::getLadybugCamera(uint32_t serialNumber) {
  printf("getLadybugCamera\n");

  LadybugCamera ladybugCamera(getDc1394Camera(serialNumber));
  return ladybugCamera;
}

dc1394camera_t* CameraContext::getDc1394Camera(uint32_t serialNumber) {

  dc1394camera_list_t* list;
  dc1394error_t error;
  error = dc1394_camera_enumerate(context, &list);
  CAMERA_EXCEPTION(error, "Failed to enumerate cameras.");

  if (list->num == 0) {
    dc1394_camera_free_list(list);
    throw CameraException(DC1394_SUCCESS, "No cameras found.");
  }

  //unsigned long oui = (list->ids[0].guid >> 40) & 0xFFFFFFUL;
  //unsigned long ser = list->ids[0].guid & 0xFFFFFFFFUL;
  for (uint32_t iCamera = 0; iCamera < list->num; iCamera++) {
    if ((serialNumber == 0) || (list->ids[iCamera].guid & 0xFFFFFFFF) == serialNumber) {
      dc1394camera_t* camera = dc1394_camera_new(context, list->ids[0].guid);
      cameras.push_back(camera);
      printf("getDc1394Camera: %u %s %u %s\n", camera->vendor_id, camera->vendor, camera->model_id, camera->model);
      dc1394_camera_free_list(list);
      return camera;
    }
  }

  dc1394_camera_free_list(list);
  throw CameraException(DC1394_SUCCESS, "Camera not found.");
}

}
}
