/*
 * LadybugCamera.cpp
 *
 *  Created on: Sep 2, 2010
 *      Author: petrito1@cmp.felk.cvut.cz
 */

#include "LadybugCamera.h"

using nifti::dc1394::Camera;

namespace nifti {
namespace dc1394 {

const uint64_t LadybugCamera::IMAGE_SELECTION_REGISTER = 0x1E88;
const uint32_t LadybugCamera::ALL_CAMERAS = 0xffffff;
const uint32_t LadybugCamera::CAMERA_0 = 0xf00000;
const uint32_t LadybugCamera::CAMERA_1 = 0x0f0000;
const uint32_t LadybugCamera::CAMERA_2 = 0x00f000;
const uint32_t LadybugCamera::CAMERA_3 = 0x000f00;
const uint32_t LadybugCamera::CAMERA_4 = 0x0000f0;
const uint32_t LadybugCamera::CAMERA_5 = 0x00000f;
const uint32_t LadybugCamera::CAMERA_0_BAYER_CHANNEL_0 = 0x800000;
const uint32_t LadybugCamera::CAMERA_0_BAYER_CHANNEL_1 = 0x400000;
const uint32_t LadybugCamera::CAMERA_0_BAYER_CHANNEL_2 = 0x200000;
const uint32_t LadybugCamera::CAMERA_0_BAYER_CHANNEL_3 = 0x100000;
const uint32_t LadybugCamera::CAMERA_1_BAYER_CHANNEL_0 = 0x080000;
const uint32_t LadybugCamera::CAMERA_1_BAYER_CHANNEL_1 = 0x040000;
const uint32_t LadybugCamera::CAMERA_1_BAYER_CHANNEL_2 = 0x020000;
const uint32_t LadybugCamera::CAMERA_1_BAYER_CHANNEL_3 = 0x010000;
const uint32_t LadybugCamera::CAMERA_2_BAYER_CHANNEL_0 = 0x008000;
const uint32_t LadybugCamera::CAMERA_2_BAYER_CHANNEL_1 = 0x004000;
const uint32_t LadybugCamera::CAMERA_2_BAYER_CHANNEL_2 = 0x002000;
const uint32_t LadybugCamera::CAMERA_2_BAYER_CHANNEL_3 = 0x001000;
const uint32_t LadybugCamera::CAMERA_3_BAYER_CHANNEL_0 = 0x000800;
const uint32_t LadybugCamera::CAMERA_3_BAYER_CHANNEL_1 = 0x000400;
const uint32_t LadybugCamera::CAMERA_3_BAYER_CHANNEL_2 = 0x000200;
const uint32_t LadybugCamera::CAMERA_3_BAYER_CHANNEL_3 = 0x000100;
const uint32_t LadybugCamera::CAMERA_4_BAYER_CHANNEL_0 = 0x000080;
const uint32_t LadybugCamera::CAMERA_4_BAYER_CHANNEL_1 = 0x000040;
const uint32_t LadybugCamera::CAMERA_4_BAYER_CHANNEL_2 = 0x000020;
const uint32_t LadybugCamera::CAMERA_4_BAYER_CHANNEL_3 = 0x000010;
const uint32_t LadybugCamera::CAMERA_5_BAYER_CHANNEL_0 = 0x000008;
const uint32_t LadybugCamera::CAMERA_5_BAYER_CHANNEL_1 = 0x000004;
const uint32_t LadybugCamera::CAMERA_5_BAYER_CHANNEL_2 = 0x000002;
const uint32_t LadybugCamera::CAMERA_5_BAYER_CHANNEL_3 = 0x000001;

const uint32_t LadybugCamera::CAMERAS[NUM_CAMERAS] = { 0xf00000, 0x0f0000, 0x00f000, 0x000f00, 0x0000f0, 0x00000f };

const int LadybugCamera::RAW_WIDTH        = 1616;
const int LadybugCamera::RAW_HEIGHT       = 1232;
const int LadybugCamera::RAW_HALF_HEIGHT  = 616;
const int LadybugCamera::RAW_NUM_IMAGES   = 6;
const int LadybugCamera::MONO_WIDTH       = 808;
const int LadybugCamera::MONO_HEIGHT      = 616;
const int LadybugCamera::MONO_HALF_HEIGHT = 308;
const int LadybugCamera::MONO_NUM_IMAGES  = 24;

void LadybugCamera::format7FrameInfo(const dc1394video_mode_t videoMode, int& frameWidth, int& frameHeight, int& imageWidth, int& imageHeight, int& numImages) {
  switch (videoMode) {
  case DC1394_VIDEO_MODE_FORMAT7_0:
  case DC1394_VIDEO_MODE_FORMAT7_6: // JPEG-compressed
  default:
    imageWidth = RAW_WIDTH;
    imageHeight = RAW_HEIGHT;
    numImages = RAW_NUM_IMAGES;
    break;
  case DC1394_VIDEO_MODE_FORMAT7_2:
    imageWidth = RAW_WIDTH;
    imageHeight = RAW_HALF_HEIGHT;
    numImages = RAW_NUM_IMAGES;
    break;
  case DC1394_VIDEO_MODE_FORMAT7_1:
  case DC1394_VIDEO_MODE_FORMAT7_7: // JPEG-compressed
    imageWidth = MONO_WIDTH;
    imageHeight = MONO_HEIGHT;
    numImages = MONO_NUM_IMAGES;
    break;
  case DC1394_VIDEO_MODE_FORMAT7_3: // JPEG-compressed
    imageWidth = MONO_WIDTH;
    imageHeight = MONO_HALF_HEIGHT;
    numImages = MONO_NUM_IMAGES;
    break;
  }

  frameWidth = imageWidth;
  frameHeight = imageHeight * numImages;
}

LadybugCamera::LadybugCamera(dc1394camera_t* camera) :
  Camera(camera), selectedImages(LadybugCamera::ALL_CAMERAS) {
}

LadybugCamera::LadybugCamera() :
  Camera(), selectedImages(LadybugCamera::ALL_CAMERAS) {
}

LadybugCamera::~LadybugCamera() {
}

void LadybugCamera::setFormat7Roi(dc1394video_mode_t videoMode, int32_t packetSize) {
  printf("LadybugCamera::setFormat7Roi(): video mode: %i, packet size: %i.\n",
      videoMode, packetSize);

  int frameWidth, frameHeight, imageWidth, imageHeight, numImages;
  format7FrameInfo(videoMode, frameWidth, frameHeight, imageWidth, imageHeight, numImages);

  Camera::setFormat7Roi(videoMode, DC1394_COLOR_CODING_RAW8, packetSize, 0, 0, frameWidth, frameHeight);
}

uint32_t LadybugCamera::getSelectedImages() {
  selectedImages = getControlRegister(IMAGE_SELECTION_REGISTER) & ALL_CAMERAS;
  return selectedImages;
}
void LadybugCamera::setSelectedImages(uint32_t images) {
  lockCapture();

  selectedImages = images;
  setControlRegister(IMAGE_SELECTION_REGISTER, selectedImages);

  // TODO: Reduce ROI to contain valid images.
//  int imageHeight = 1232;
//  int frameHeight = getNumSelectedImages() * imageHeight;
//  Camera::setFormat7Roi(DC1394_VIDEO_MODE_FORMAT7_0, DC1394_COLOR_CODING_RAW8, 9792, 0, 0, 1616, frameHeight);

  unlockCapture();
}
bool LadybugCamera::areImagesSelected(uint32_t images) {
  //  return (getSelectedImages() & images) == images;
  return (selectedImages & images) == images;
}
int LadybugCamera::getNumSelectedImages() {
  // TODO: Fix for mono images.
  int numImages = 0;
  for (int iCam = 0; iCam < NUM_CAMERAS; iCam++) {
    if (areImagesSelected(CAMERAS[iCam])) {
      numImages++;
    }
  }
  return numImages;
}

}
}
