/*
 * raw_image_subscriber.cpp
 *
 *  Created on: Aug 31, 2010
 *      Author: petrito1@cmp.felk.cvut.cz
 */

#include <cv_bridge/CvBridge.h>
#include <image_transport/image_transport.h>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <ros/ros.h>

namespace {
std::string nodeName = "raw_image_subscriber";
std::string imageTopic = "raw";
}

//void imageCallback(const sensor_msgs::ImagePtr &msg) {
void imageCallback(const sensor_msgs::ImageConstPtr &msg) {

  ROS_DEBUG("Message received: size: %dx%d, step: %d, encoding: %s", msg->width, msg->height, msg->step,
      msg->encoding.data());

  sensor_msgs::CvBridge bridge;
//  IplImage* iplImage = bridge.imgMsgToCv(msg, "passthrough");
    IplImage* iplImage = bridge.imgMsgToCv(msg, "mono8");

  cv::Mat inImage(iplImage);
  ROS_DEBUG("In image: size: %dx%d, step: %d, type: %d", inImage.cols, inImage.rows, inImage.step, inImage.type());

  cv::Mat outImage;
  // The output RGB components of a pixel are interpolated from 1, 2 or 4 neighbors of the pixel having the same color.
  cv::cvtColor(inImage, outImage, CV_BayerBG2BGR);
  ROS_DEBUG("Out image: size: %dx%d, step: %d, type: %d", outImage.cols, outImage.rows, outImage.step, outImage.type());
  cv::Mat small;
  cv::resize(outImage, small, cv::Size(), 0.125, 0.125);
  cv::imshow(nodeName, small);
}

int main(int argc, char **argv) {

  ros::init(argc, argv, nodeName);
  ros::NodeHandle nh;
  image_transport::ImageTransport it(nh);
  cv::namedWindow(nodeName);
  cvStartWindowThread();
  image_transport::Subscriber sub = it.subscribe(imageTopic, 1, imageCallback);
  ros::spin();
  cvDestroyWindow(nodeName.data());
}

