/*
 * static_virtual_camera.cpp
 *
 *  Created on: Aug 23, 2010
 *      Author: petrito1@cmp.felk.cvut.cz
 *
 * Test program for LadybugVirtualCamera.
 * Loads mesh, alphamasks and images, and outputs rendered virtual view image.
 *
 * Usage: static_virtual_camera <mesh file> <alphamask prefix> <image prefix>
 */

#include <stdio.h>

#include <GL/glew.h>
#include <GL/gl.h>
#include <GL/glut.h>
#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>

#include "LadybugVirtualCamera.h"

using std::string;

using cv::Mat;

using nifti::ladybug::LadybugVirtualCamera;

int main(int argc, char **argv) {
  if (argc < 4) {
    return -1;
  }

  glutInit(&argc, argv);

  string meshFile = string(argv[1]);
  string alphamaskPrefix = string(argv[2]);
  string imagePrefix = string(argv[3]);

  LadybugVirtualCamera virtualCamera;
  virtualCamera.loadMesh(meshFile);
  virtualCamera.loadAlphamask(alphamaskPrefix);

  const int nCams = 6;
  for (int iCam = 0; iCam < nCams; iCam++) {
    char imagePathBuffer[128];
    sprintf(imagePathBuffer, "%s%d%s", imagePrefix.data(), iCam, ".ppm");
    Mat image = cv::imread(string(imagePathBuffer));
    virtualCamera.setImage(iCam, image);
  }

  Mat view;
  virtualCamera.renderView(view);
  cv::imwrite("view.png", view);
}

