/*
 * test_ladybug_roi.cpp
 *
 *  Created on: Sep 2, 2010
 *      Author: petrito1@cmp.felk.cvut.cz
 */

#include <opencv2/core/core.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/highgui/highgui.hpp>

#include <signal.h>
#include <sys/param.h>
#include <sys/times.h>

#include "CameraException.h"
#include "Camera.h"
#include "CameraContext.h"

#include "LadybugVirtualCamera.h"

#define IMAGE_SELECTION_REGISTER 0x1E88

#define IMAGE_SELECTION_CAMERA0_CHANNEL0_MASK (1 << 23)
#define IMAGE_SELECTION_CAMERA1_CHANNEL0_MASK (1 << 19)
#define IMAGE_SELECTION_CAMERA2_CHANNEL0_MASK (1 << 15)
#define IMAGE_SELECTION_CAMERA3_CHANNEL0_MASK (1 << 11)
#define IMAGE_SELECTION_CAMERA4_CHANNEL0_MASK (1 << 7)
#define IMAGE_SELECTION_CAMERA5_CHANNEL0_MASK (1 << 3)

using std::string;

using cv::Mat;

using nifti::dc1394::Camera;
using nifti::dc1394::CameraContext;
using nifti::dc1394::CameraException;
using nifti::dc1394::LadybugCamera;

void sigintHandler(int sig);
void printDuration(char* message, tms t1, long r1);

namespace {
bool halt = false;
}

int main(int argc, char **argv) {

  signal(SIGINT, sigintHandler);

  std::string winName = "ladybug_image";
  cv::namedWindow(winName);
//  cv::startWindowThread();
  cvStartWindowThread();

  CameraContext context;
  LadybugCamera camera = context.getLadybugCamera(10095209);
  try {
    //  camera.resetBus();
    camera.reset();
    //  camera.setup();
    camera.setOperationMode(DC1394_OPERATION_MODE_1394B);
    camera.setIsoSpeed(DC1394_ISO_SPEED_800);
    camera.setVideoMode(DC1394_VIDEO_MODE_FORMAT7_0);
    int fullHeight = 7392;
    int imageHeight = fullHeight / 6;
    int cam = 0;
    //  camera.setFormat7Roi(DC1394_VIDEO_MODE_FORMAT7_0, DC1394_COLOR_CODING_RAW8, 9792, 0, 0, 1616, 7392);
    //  camera.setFormat7Roi(DC1394_VIDEO_MODE_FORMAT7_0, DC1394_COLOR_CODING_RAW8, 8192, 0, cam * imageHeight, 1616, imageHeight);
    //    camera.setFormat7Roi(DC1394_VIDEO_MODE_FORMAT7_0, DC1394_COLOR_CODING_RAW8, 9792, 0, 0, 1616, 2 * imageHeight);
    camera.setFormat7Roi(DC1394_VIDEO_MODE_FORMAT7_0, DC1394_COLOR_CODING_RAW8, 9792, 0, 0, 1616, imageHeight);

    //    camera.setControlRegister(IMAGE_SELECTION_REGISTER, (1 << 7) | (1 << 3));
    //    camera.setControlRegister(IMAGE_SELECTION_REGISTER, 0xf);
    //    camera.setControlRegister(IMAGE_SELECTION_REGISTER, (1 << 3));
    //    camera.setControlRegister(IMAGE_SELECTION_REGISTER, (1 << 7));

//    camera.setImagesSelected(LadybugCamera::CAMERA_5);

    uint32_t imageSelectionValue = camera.getControlRegister(IMAGE_SELECTION_REGISTER);
    printf("Image selection presence: %u\n", (imageSelectionValue & (1 << 31)) >> 31);

    int iCamera = 0;
    int iBayerChannel = 0;
    for (int offset = 23; offset >= 0; offset--) {
      printf("Image selection: camera %d: bayer channel %d: %u\n", iCamera, iBayerChannel, (imageSelectionValue & (1
          << offset)) >> offset);
      iBayerChannel++;
      if (iBayerChannel > 3) {
        iCamera++;
        iBayerChannel = 0;
      }
    }

    printf("Format 7 color coding: %d.\n", camera.getFormat7ColorCoding());
    printf("Format 7 color filter: %d.\n", camera.getFormat7ColorFilter());
    printf("Format 7 frame interval: %f.\n", camera.getFormat7FrameInterval());
    printf("Format 7 packets per frame: %u.\n", camera.getFormat7PacketsPerFrame());
    printf("Format 7 packet size: %u.\n", camera.getFormat7PacketSize());
    printf("Format 7 recommended packet size: %u.\n", camera.getFormat7RecommendedPacketSize());

    camera.setupCapture();
    camera.startVideoTransmission();

    struct tms t1;
    long r1;

    int iFrame = 0;
    Mat frame;
    Mat small;
    while (!halt) {
      // Cycle through cameras.
      switch (iFrame % 50) {
      case 0:
        camera.setSelectedImages(LadybugCamera::CAMERA_0);
        break;
      case 10:
        camera.setSelectedImages(LadybugCamera::CAMERA_1);
        break;
      case 20:
        camera.setSelectedImages(LadybugCamera::CAMERA_2);
        break;
      case 30:
        camera.setSelectedImages(LadybugCamera::CAMERA_3);
        break;
      case 40:
        camera.setSelectedImages(LadybugCamera::CAMERA_4);
        break;
      }

      r1 = times(&t1);
      camera.captureColorMat(frame, true);
      printDuration("Frame capture:     ", t1, r1);

      r1 = times(&t1);
      cv::resize(frame, small, cv::Size(), 0.25, 0.25, CV_INTER_NN);
      printDuration("Image resize:      ", t1, r1);

      r1 = times(&t1);
      cv::imshow(winName, small);
      printDuration("Image display:     ", t1, r1);

      iFrame++;
    }
  } catch (...) {
  }

  camera.stopVideoTransmission();
  camera.stopCapture();
  camera.releaseAll();

//  cv::destroyWindow(winName);
  cvDestroyWindow(winName.data());
}

void sigintHandler(int sig) {
  printf("Signal %d received. Halting...\n", sig);
  halt = true;
}

void printDuration(char* message, tms t1, long r1) {
  struct tms t2;
  long r2;
  r2 = times(&t2);

  printf("%s", message);
  printf("user = %1.2f, ", ((float) (t2.tms_utime - t1.tms_utime)) / HZ);
  printf("system = %1.2f, ", ((float) (t2.tms_stime - t1.tms_stime)) / HZ);
  printf("real = %1.2f\n", ((float) (r2 - r1)) / HZ);
}
